<?php

namespace Drupal\zohomail\Plugin\Mail;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Mail\MailInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Component\Utility\Html;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\zohomail\Helper\ZohoMailApi;
use Drupal\Core\Mail\MailFormatHelper;
/**
 * Modify the Drupal mail system to use Zoho Mail when sending emails.
 *
 * @Mail(
 *   id = "zohomail_mail",
 *   label = @Translation("Zoho Mail mailer"),
 *   description = @Translation("Sends the message using Zoho Mail.")
 * )
 */
class ZohoMail implements MailInterface, ContainerFactoryPluginInterface {

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

 
  public function __construct(ImmutableConfig $settings, LoggerInterface $logger) {
    $this->config = $settings;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('config.factory')->get('zohomail.settings'),
      $container->get('logger.factory')->get('zohomail')
    );
  }

  /**
   * Concatenate and wrap the e-mail body for either plain-text or HTML e-mails.
   *
   * @param array $message
   *   A message array, as described in hook_mail_alter().
   *
   * @return array
   *   The formatted $message.
   */
  public function format(array $message) {
    // Join the body array into one string.
    if (is_array($message['body'])) {
      $message['body'] = implode("\n\n", $message['body']);
    }

    // If text format is specified in settings, run the message through it.
    $content_type = $message['headers']['Content-Type'];
      
    if (strpos($content_type, 'text/plain') !== FALSE) {
      $message['body'] =  MailFormatHelper::htmlToText($message['body']);
    }

    return $message;
  }

 
  public function mail(array $message) {

    $domain = $this->config->get('domain');
		$account_id = $this->config->get('account_id');

    if(!isset($account_id)) {
      return false;
    }
		

    $zmailApi = new ZohoMailApi($this->config);
    $headers = $message['headers'];
    $to = array();
		

    $mail_data = [
      'subject' => $message['subject']
    ];

    $mail_data['fromAddress'] = $this->config->get('from_address');
    
    $fromAddress =  [
			'name' => $this->config->get('from_name'),
			'address' => $this->config->get('from_address')
			];
      
    if ($message['to'] !== null) {
      $mail_data['toAddress'] = $message['to'];
    }

    if (!empty($message['headers']['Content-Type'])) {
      $content_type = $message['headers']['Content-Type'];
      if (strpos($content_type, 'text/plain') !== FALSE) {
        $mail_data['mailFormat'] = 'plaintext';
        $mail_data['content'] = $message['body'];
      } 
      else {
        $mail_data['mailFormat'] = 'html';
        $mail_data['content'] = $message['body'];
      }
    } else {
      $mail_data['mailFormat'] = 'html';
      $mail_data['content'] = $message['body'];
    }
    
    if (!empty($headers['Cc'])) {
      $mail_data['ccAddress'] = $headers['Cc'];
    }
    if (!empty($headers['Bcc'])) {
      $mail_data['bccAddress'] = $headers['Bcc'];
    }
    if (!empty($headers['Return-Path'])) {
      $allowed_emails = (array)$this->config->get('allowed_emails');
      if(isset($allowed_emails) &&  in_array($headers['Return-Path'],$allowed_emails))
      {
        $mail_data['replyTo'] = $headers['Return-Path'];
      }
      
    }
    
    if (!empty($message['params']['attachments'])) {
      $attachmentRes = $zmailApi->uploadAttachment($message['params']['attachments']);
			$mail_data['attachments'] = json_decode($attachmentRes)->data;
    }

$responseObj = $zmailApi->sendMail($mail_data);

if($responseObj->result !== "success") {
  $this->logger->error(json_encode($responseObj));
  return false;
}
return true;
    
  }

  private function splitNameAndAddress($emailAddress) {
		$result = [
			'name' => '',
			'address' => ''
			];
		$pos = strpos($emailAddress, '<');
		if($pos !== false) {
			$result['address'] =  substr($emailAddress, $pos+1, strlen($emailAddress)-$pos-2);
			$result['name'] = substr($emailAddress,0,$pos-1);
		} else {
			$result['address'] = $emailAddress;
		}
		
		return $result;
	}
	private function getEmailDetails($emailAddressDetails) {
		if(!is_array($emailAddressDetails)){
			$emailAddressDetails = explode( ',', $emailAddressDetails );
		}
		$result = array();
		
		
		foreach($emailAddressDetails as $emailAddress){
			$emailDetail = array();
			$emailDetail["email_address"] = $this::splitNameAndAddress($emailAddress);
			array_push($result,$emailDetail);
		}
		
		return $result;
	}

}
